

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedHashMap;

import simulation.Impl.SimulationRMIXMLDocumentObserver;
import simulation.Impl.SimulationVisualizationObserver;
import simulation.Impl.SimulationWorldState;

import massim.DefaultObserver;
import massim.SimulationConfiguration;
import massim.SimulationState;
import massim.server.ServerSimulationContext;
import massim.server.ServerSimulationContextReceiver;
import massim.simulation.simplesimulation.AgentState;
import massim.simulation.simplesimulation.SimulationStateImpl;
import massim.visualization.*;
//import massim.server.*;

/**
 * This VisualizationObserver takes care about the visualization (svg-files).
 * 
 * <<< Based on the implementation of Michael K�ster (Tournament 2006) >>>
 * 
 */

public class GridSimulationVisualizationObserver extends SimulationVisualizationObserver{
	
	public GridSimulationVisualizationObserver() {
		super();
		
	}	
	@Override
	protected synchronized void drawSimulation(int step, SimulationWorldState s_state, AgentState[] agentstates) {
		GridSimulationWorldState state = (GridSimulationWorldState)s_state;
		// draw grid
		output.create();
		output.drawGrid(0, state.sizex, state.sizey);

		for (int x = 0; x < state.sizex; x++) {
			for (int y = 0; y < state.sizey; y++) {
			
				 //draw obstacles
				if (state.board[x][y].obstacle) {
					output.drawTrees(0, x, y);
				}
				 
				// draw items

				if (state.board[x][y].stable1)
				{
					output.drawStable(0, x, y, false);
				}
				if (state.board[x][y].stable2)
				{
					output.drawStable(1, x, y, true);
				}
				if (state.board[x][y].cow) {
					output.drawCow(0, x, y);
				}
				if (state.board[x][y].fence && !state.board[x][y].open) {
					output.drawFences(x, y);
					
				}
				if(state.board[x][y].switcher){
					output.drawSwitch(x, y);
				}
			}
		}

		// draw agents and statistic-output
		// general statistics
		
		LinkedHashMap<String, String> statistic = new LinkedHashMap<String, String>();
		statistic.put("Step", state.currentStep.toString());
		statistic.put(state.teamName[0] + " Color: ", " red");
		statistic.put(state.teamName[1] + " Color: ",  "blue");
		statistic.put(state.teamName[0] + " Score", state.teamScore[0].toString());
		statistic.put(state.teamName[1] + " Score", state.teamScore[1].toString());
		//statistic.put("Cows", Integer.toString(state.numberOfAgents) );
		statistic.put("Size", "(" + state.sizex + "," + state.sizey + ")");

		output.setHeadInformationFirstLevel(tournamentName);
		output.setHeadInformationSecondLevel(simulationName);

		for (Integer i = 0; i < agentstates.length; i++) {		
			// agents
			String color;
			GridSimulationAgentState agentstate = (GridSimulationAgentState)agentstates[i];
			
			if (agentstate.team == state.teamName[0]) {
				
					color = "red";
			//		output.drawCircle("agentred", agentstate.posx, agentstate.posy, "red");
			}
			
			else{
					color = "blue";
					//output.drawCircle("agentblue", agentstate.posx, agentstate.posy, "blue");
			}			
		//		output.drawGoldDigger(i, color, agentstate.posx,
		//		agentstate.posy);
				output.drawAgent(color,agentstate.posx,agentstate.posy);
			
			// statistics
		//	statistic.put(i + ":" + "Name", agentstate.name);
			
		//	statistic.put(i.toString(), agentstate.actionFailureProbability + "%,   " + agentstate.lastAction);
		}
		
		output.setGridStatistic(statistic);
		
		laststep=step;
		
		output.save();
		
	}
}
