package simulation.Impl;
import static massim.util.DebugLog.LOGLEVEL_DEBUG;
import static massim.util.DebugLog.log;
import massim.SimulationConfiguration;
import massim.SimulationState;
import massim.XMLOutputObserver;
import massim.backup.BackupReader;
import massim.backup.BackupWriter;
import massim.rmi.RMI_Infor;
import massim.server.ServerSimulationContext;
import massim.server.ServerSimulationContextReceiver;
import massim.simulation.simplesimulation.SimulationStateImpl;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * * This XMLObserver provides the simulation statistics.
 * 
 * <<< Based on the implementation of Michael Köster (Tournament 2006) >>>
 * 
 * @author Slawomir Deren
 * 
 */
public class SimulationXMLStatisticsObserver extends XMLOutputObserver{
	
	private Element el_root;
	private Document doc;
	
	// private int[] agentscores;
	private String[] teamName;
    public static String outputFolder="/home/massim/www/webapps/massim/output/";
	private int[] teamScore;
	public static String simulationName;
	public  String tournamentName ;
	private String configFile = "conf" + System.getProperty("file.separator")
	+ "visualconfig.xml";

	/**
	 * This constructor appends the statistic-element to the document.
	 */
	public SimulationXMLStatisticsObserver() {
		super();
		doc = getDocument();
		el_root = doc.createElement("statistics");
		Document doc = BackupReader.openFile(configFile);
		Element el = (Element) doc.getElementsByTagName("simulationOutput").item(0);
		outputFolder = el.getAttribute("path");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see massim.Observer#notifySimulationStart()
	 */
	public void notifySimulationStart() {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see massim.Observer#notifySimulationEnd()
	 */
	public void notifySimulationEnd() {
		
		synchronized (this) {
			resetDocument();
			doc = getDocument();
			el_root = doc.createElement("statistics");
			el_root.setAttribute(teamName[0], Integer.toString(teamScore[0]));
			el_root.setAttribute(teamName[1], Integer.toString(teamScore[1]));
			el_root.setAttribute("simulation", simulationName);
		
			String outputFile;
			
			if(RMI_Infor.FLASH_SERVER_ACTIVATED){
				outputFile = outputFolder+simulationName+BackupWriter.file_sep+"CompleteGame.swf";
			}
			else{
				outputFile = outputFolder+simulationName+BackupWriter.file_sep+"SimulationPreview.svg";
			}
			el_root.setAttribute("output",outputFile);
			
			doc.appendChild(el_root);
		}

		setChanged();
		notifyObservers();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see massim.Observer#notifySimulationState(massim.SimulationState)
	 */
	public void notifySimulationState(SimulationState state) {
		
		SimulationStateImpl simplestate = (SimulationStateImpl) state;
		SimulationWorldState worldState = (SimulationWorldState) simplestate.simulationState;
		
		teamName = new String[2];
		teamScore = new int[2];
		teamName[0] = worldState.teamName[0];
		teamName[1] = worldState.teamName[1];
		teamScore[0] = worldState.teamScore[0];
		teamScore[1] = worldState.teamScore[1];
		
		
		String outputFile;
		if(RMI_Infor.FLASH_SERVER_ACTIVATED){
			simulationName  = worldState.simulationName;
			outputFile = outputFolder+simulationName+BackupWriter.file_sep+"masSim-"+simplestate.steps+".swf";
		}
		else{
			outputFile = outputFolder+simulationName+BackupWriter.file_sep+"masSim-"+simplestate.steps+".svg";
		}
		
		synchronized (this) {
			resetDocument();
			doc = getDocument();
			el_root = doc.createElement("statistics");
			el_root.setAttribute(teamName[0], Integer.toString(teamScore[0]));
			el_root.setAttribute(teamName[1], Integer.toString(teamScore[1]));
			el_root.setAttribute("simulation", simulationName);
			el_root.setAttribute("output",outputFile);
			doc.appendChild(el_root);
		}

		setChanged();
		notifyObservers();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see massim.Component#start()
	 */
	public void start() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see massim.Component#stop()
	 */
	public void stop() {
	}
}